/*********************************************************************
 *
 *       Run Time Library Loading(RTLL)- System Loader Framework         
 *         
 *
 *********************************************************************
 * FileName:        system_loader.c
 * Dependencies:
 * Processor:       PIC32
 *
 * Complier:        MPLAB C32
 *                  MPLAB IDE
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 * Microchip Audio Library  PIC32 Software.
 * Copyright  2008 Microchip Technology Inc.  All rights reserved.
 * 
 * Microchip licenses the Software for your use with Microchip microcontrollers
 * and Microchip digital signal controllers pursuant to the terms of the
 * Non-Exclusive Software License Agreement accompanying this Software.
 *
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY
 * OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION,
 * ANY WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * MICROCHIP AND ITS LICENSORS ASSUME NO RESPONSIBILITY FOR THE ACCURACY,
 * RELIABILITY OR APPLICATION OF THE SOFTWARE AND DOCUMENTATION.
 * IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED
 * UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH
 * OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
 * DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL,
 * SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS
 * OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY,
 * SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED
 * TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *
 ********************************************************************/


#ifdef __RTLL_APP_SERVICES__

/***********************************************************************************
      Following code sections are compiled for application only.
*************************************************************************************/
#include <string.h>
#include "app_services.h"

/********************************************************************
* Function: 	dlopen()
*
* Precondition: 
*
* Input: 		*libString: Pointer to Library/Module Name.
				*libLoadPoint: Address of the library/module header.
*
* Output:		Handle to library header.
*
* Side Effects:	None.
*
* Overview:      Does following things.
				* Checks if library is loaded.
				* Calls the library start up code, if library is found.
				* Registers application's memory management functions 
				  with the library. We want library to use application's 
				  heap.
*
*			
* Note:		 .
********************************************************************/
void* dlopen(char *libString, void* libLoadPoint)
{
	
	const T_MODULE_DYN_HDR* lib = (const T_MODULE_DYN_HDR*)libLoadPoint;
	
	if(!strcmp(lib->module_name, libString))
	{
		// Got the library we are looking for
		// Call the start up code and export heap management functions to library.
		lib->libInit((void*)&_mmCallBack);
		return((void*)lib);	
	}	
	// Failed to get the library.
	return 0;	
}	


/********************************************************************
* Function: 	dlsym()
*
* Precondition: 
*
* Input: 		*libHandle: Handle to the library header.
				*funcName: Proc/func name string.
*
* Output:		Proc Address.
*
* Side Effects:	None.
*
* Overview:      Does following things.
				* Checks if library has the function/procedure.
				*Returns the function/procedure handle if the
				string is found in the export proc table of the
				library.
*
*			
* Note:		 	This handle must be called by the application 
				to initialize the library.
********************************************************************/
void*  dlsym(void* libHandle, char* funcName)
{
	const T_MODULE_DYN_HDR* libPtr = (const T_MODULE_DYN_HDR*)libHandle;
	int i;
	
	
	for(i = 0; i < libPtr->nProcs; i++)
	{
		if(!strcmp(libPtr->procTbl[i].procString, funcName))
		{
			return ((void *)libPtr->procTbl[i].proc);	
		}	
		
	}		
	return 0;	
}	


#endif //__RTLL_APP_SERVICES__



/*******************************************************************************************************************************************************/
		// Following code sections are compiled only for library		
		
/********************************************************************************************************************************************************/

#ifdef __RTLL_LIB_SERVICES__
#include "lib_services.h"

#define _nProcs (sizeof(_exportProcTbl)/sizeof(T_PROC_DCPT))

void _libInit(void* callBkFnPtrTbl);

//Variable to store the memory management call back 
//function pointers
static T_LIB_CALL_BACK* _mmCallBack;




// Standard header which must be exposed by the library.
//#if (__C32_VERSION__ >= 200)
const T_MODULE_DYN_HDR __attribute__((address(_MODULE_HEADER_ADDRESS_))) _ModuleLoadHdr =
//#else
//const T_MODULE_DYN_HDR __attribute__((__section__(".Module_Header_Section"))) _ModuleLoadHdr =*/
//#endif
{
	
	_MODULE_NAME_,			// name of the Module	
	_libInit,			    // Start up code
	_nProcs,				// Number of procedures
	_exportProcTbl		    // Export Procedure Table	
};	

extern void _initLibVariables(void);
/********************************************************************
* Function: 	dl_libInit()
*
* Precondition: 
*
* Input: 		Exported heap management function pointer table.
*
* Output:		None.
*
* Side Effects:	None.
*
* Overview:      Does following things.
				* All uninitialized variables are set to 0 (BSS section).
				* Initialize, initialized variables.
*
*			
* Note:		 	This handle must be called by the application 
				to initialize the library.
********************************************************************/
void _libInit(void* callBkFnPtrTbl)
{
	_initLibVariables();
	// Call any user specific Initialization.
	LibInit();	
}

#ifdef USE_APPLICATION_HEAP
/********************************************************************
* Function: 	free()
*
* Precondition: 
*
* Input: 		mem pointer.
*
* Output:		None.
*
* Side Effects:	None.
*
* Overview:     We are overriding the "free()" function provided by the 
				compiler library <stdlib>. Instead, we will be calling 
				application's free() here.
*
*			
* Note:		 	
********************************************************************/
void free(void* ptr)
{
	if(_mmCallBack->freePtr != 0)
	{
		_mmCallBack->freePtr(ptr);
	}	
	else
	{
		//error
		while(1);
	}		
}	


/********************************************************************
* Function: 	malloc()
*
* Precondition: 
*
* Input: 		mem size.
*
* Output:		mem pointer.
*
* Side Effects:	None.
*
* Overview:     We are overriding the "malloc()" provided by the 
				compiler library <stdlib>. Instead, we will be calling 
				application's "malloc()" here.
*
*			
* Note:		 	
***********************************************************************/	
void* malloc(size_t n)
{
	if(_mmCallBack->mallocPtr != 0)
	{
		return(_mmCallBack->mallocPtr(n));
	}
	else
	{
		return 0;
	}		
}	

#endif	

#endif //__RTLL_LIB_SERVICES__	

/********************end***********************************************/


